=== modified file 'bzrlib/commands.py'
--- bzrlib/commands.py	2006-06-10 13:27:24 +0000
+++ bzrlib/commands.py	2006-06-11 02:46:51 +0000
@@ -41,6 +41,7 @@
                            BzrOptionError,
                            NotBranchError)
 from bzrlib.option import Option
+import bzrlib.osutils
 from bzrlib.revisionspec import RevisionSpec
 from bzrlib.symbol_versioning import *
 import bzrlib.trace
@@ -235,17 +236,7 @@
             self.outf = sys.stdout
             return
 
-        output_encoding = getattr(sys.stdout, 'encoding', None)
-        if not output_encoding:
-            input_encoding = getattr(sys.stdin, 'encoding', None)
-            if not input_encoding:
-                output_encoding = bzrlib.user_encoding
-                mutter('encoding stdout as bzrlib.user_encoding %r', output_encoding)
-            else:
-                output_encoding = input_encoding
-                mutter('encoding stdout as sys.stdin encoding %r', output_encoding)
-        else:
-            mutter('encoding stdout as sys.stdout encoding %r', output_encoding)
+        output_encoding = bzrlib.osutils.get_terminal_encoding()
 
         # use 'replace' so that we don't abort if trying to write out
         # in e.g. the default C locale.

=== modified file 'bzrlib/osutils.py'
--- bzrlib/osutils.py	2006-06-11 02:22:55 +0000
+++ bzrlib/osutils.py	2006-06-11 02:46:51 +0000
@@ -285,6 +285,33 @@
         return shutil.rmtree(path, ignore_errors, onerror)
 
 
+def get_terminal_encoding():
+    """Find the best encoding for printing to the screen.
+
+    This attempts to check both sys.stdout and sys.stdin to see
+    what encoding they are in, and if that fails it falls back to
+    bzrlib.user_encoding.
+    The problem is that on Windows, locale.getpreferredencoding()
+    is not the same encoding as that used by the console:
+    http://mail.python.org/pipermail/python-list/2003-May/162357.html
+
+    On my standard US Windows XP, the preferred encoding is
+    cp1252, but the console is cp437
+    """
+    output_encoding = getattr(sys.stdout, 'encoding', None)
+    if not output_encoding:
+        input_encoding = getattr(sys.stdin, 'encoding', None)
+        if not input_encoding:
+            output_encoding = bzrlib.user_encoding
+            mutter('encoding stdout as bzrlib.user_encoding %r', output_encoding)
+        else:
+            output_encoding = input_encoding
+            mutter('encoding stdout as sys.stdin encoding %r', output_encoding)
+    else:
+        mutter('encoding stdout as sys.stdout encoding %r', output_encoding)
+    return output_encoding
+
+
 def normalizepath(f):
     if hasattr(os.path, 'realpath'):
         F = realpath

=== modified file 'bzrlib/tests/test_osutils.py'
--- bzrlib/tests/test_osutils.py	2006-06-08 16:12:13 +0000
+++ bzrlib/tests/test_osutils.py	2006-06-11 02:46:51 +0000
@@ -25,7 +25,12 @@
 import bzrlib
 from bzrlib.errors import BzrBadParameterNotUnicode, InvalidURL
 import bzrlib.osutils as osutils
-from bzrlib.tests import TestCaseInTempDir, TestCase, TestSkipped
+from bzrlib.tests import (
+        StringIOWrapper,
+        TestCase, 
+        TestCaseInTempDir, 
+        TestSkipped,
+        )
 
 
 class TestOSUtils(TestCaseInTempDir):
@@ -259,3 +264,42 @@
         self.assertTrue(found_bzrdir)
         self.assertEqual(expected_dirblocks,
             [[line[0:3] for line in block] for block in result])
+
+
+class TestTerminalEncoding(TestCase):
+    """Test the auto-detection of proper terminal encoding."""
+
+    def setUp(self):
+        self._stdout = sys.stdout
+        self._stderr = sys.stderr
+        self._stdin = sys.stdin
+        self._user_encoding = bzrlib.user_encoding
+
+        self.addCleanup(self._reset)
+
+        sys.stdout = StringIOWrapper()
+        sys.stdout.encoding = 'stdout_encoding'
+        sys.stderr = StringIOWrapper()
+        sys.stderr.encoding = 'stderr_encoding'
+        sys.stdin = StringIOWrapper()
+        sys.stdin.encoding = 'stdin_encoding'
+        bzrlib.user_encoding = 'user_encoding'
+
+    def _reset(self):
+        sys.stdout = self._stdout
+        sys.stderr = self._stderr
+        sys.stdin = self._stdin
+        bzrlib.user_encoding = self._user_encoding
+
+    def test_get_terminal_encoding(self):
+        # first preference is stdout encoding
+        self.assertEqual('stdout_encoding', osutils.get_terminal_encoding())
+
+        sys.stdout.encoding = None
+        # if sys.stdout is None, fall back to sys.stdin
+        self.assertEqual('stdin_encoding', osutils.get_terminal_encoding())
+
+        sys.stdin.encoding = None
+        # and in the worst case, use bzrlib.user_encoding
+        self.assertEqual('user_encoding', osutils.get_terminal_encoding())
+

